@@ -144,7 +144,7 @@ use Class::Adapter ();
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '1.04';
+	$VERSION = '1.06';
 }
 
 
@@ -155,23 +155,27 @@ BEGIN {
 # Constructor
 
 sub new {
-	my $class = ref $_[0] ? ref shift : shift;
-	bless {
-		target  => shift,
+	my $class = ref $_[0] || $_[0];
+	return bless {
+		target  => $_[1],
 		isa     => [ 'Class::Adapter' ],
 		modules => {},
 		methods => {},
-		}, $class;
+	}, $class;
 }
 
 sub import {
-	my $class  = shift;
-	return 1 unless @_; # Must have at least one param
+	my $class = shift;
+
+	# Must have at least one param
+	return 1 unless @_;
 
 	# Create the Builder object
 	my $target = caller;
-	my $self   = $class->new( $target )
-		or Carp::croak("Failed to create Class::Adapter::Builder object");
+	my $self   = $class->new( $target );
+	unless ( $self ) {
+		Carp::croak("Failed to create Class::Adapter::Builder object");
+	}
 
 	# Process the option pairs
 	while ( @_ ) {
@@ -193,12 +197,20 @@ sub import {
 	# Generate the code
 	my $code = $self->make_class or Carp::croak(
 		"Failed to generate Class::Adapter::Builder class"
-		);
+	);
+
+	# Compile the combined code via a temp file so that debugging works
+	#require File::Temp;
+	#my ($fh, $filename) = File::Temp::tempfile();
+	#$fh->print("$code");
+	#close $fh;
+	#require $filename;
+	#print "Loaded '$filename'\n";
 
 	eval "$code";
 	$@ and Carp::croak(
-		"Error while initialising Class::Adapter::Builder class '$target'"
-		);
+		"Error while compiling Class::Adapter::Builder class '$target' ($@)"
+	);
 
 	$target;
 }
@@ -211,17 +223,28 @@ sub import {
 # Main Methods
 
 sub set_NEW {
-	my $self = shift;
+	my $self     = shift;
 	$self->{new} = shift;
+
+	# We always need Scalar::Util to pass through new
 	$self->{modules}->{'Scalar::Util'} = 1;
-	1;
+
+	# Add a use for the module unless it is already loaded.
+	# We test with the can call instead of just blindly require'ing in
+	# case we want to NEW to something that doesn't have it's own
+	# .pm file. 
+	unless ( $self->{new}->can('new') ) {
+		$self->{modules}->{ $self->{new} } = 1;
+	}
+
+	return 1;
 }
 
 sub set_ISA {
 	my $self  = shift;
 	my $array = ref $_[0] eq 'ARRAY' ? shift : [ @_ ];
 	$self->{isa} = $array;
-	1;
+	return 1;
 }
 
 sub set_AUTOLOAD {
@@ -235,16 +258,16 @@ sub set_AUTOLOAD {
 	} else {
 		delete $self->{autoload};
 	}
-	1;
+	return 1;
 }
 
 sub set_METHODS {
-	my $self = shift;
+	my $self  = shift;
 	my $array = ref $_[0] eq 'ARRAY' ? shift : [ @_ ];
 	foreach my $name ( @$array ) {
 		$self->set_method( $name, $name ) or return undef;
 	}
-	1;
+	return 1;
 }
 
 sub set_method {
@@ -256,27 +279,35 @@ sub set_method {
 	} else {
 		return undef;
 	}
-	1;
+	return 1;
 }
 
 
 
 
 
-
-
-
 #####################################################################
 # Code Generation Functions
 
 sub make_class {
 	my $self  = shift;
 
-	# Biuld up the parts of the class
+	# Generate derived lists
+	my %seen  = ();
+	$self->{load} = [
+		grep { $_ !~ /^Class::Adapter(?:::Builder)?$/ }
+		sort grep { ! $seen{$_}++ }
+		keys %{$self->{modules}}
+	];
+	$self->{fake} = [
+		grep { ! $seen{$_} } grep { $_ ne '_OBJECT_' } @{$self->{isa}}
+	];
+
+	# Build up the parts of the class
 	my @parts = (
 		"package $self->{target};\n\n"
 		. "# Generated by Class::Abstract::Builder\n"
-		);
+	);
 
 	if ( keys %{$self->{modules}} ) {
 		push @parts, $self->_make_modules;
@@ -303,21 +334,31 @@ sub make_class {
 		push @parts, $self->_make_AUTOLOAD( $self->{target}, $self->{autoload_public} );
 	}
 
-	join( "\n", @parts, "1;\n" );
+	return join( "\n", @parts, "1;\n" );
 }
 
-
-
-
 sub _make_modules {
-	my $self = shift;
-	my $str = join '', map { "use $_ ();\n" }
-		grep { $_ !~ /^Class::Adapter(?:::Builder)?$/ }
-		sort keys %{$self->{modules}};
-	"use strict;\n${str}use base 'Class::Adapter';\n";
+	my $self  = shift;
+	my $pkg   = $self->{target};
+	my $load  = join '',
+		map { "\nuse $_ ();" }
+		@{$self->{load}};
+
+	# Foo->isa('Foo') returns false if the namespace does not exist
+	# Use the package command in a scope to create namespaces where needed.
+	my $namespaces = join '',
+		map { "\n\t$_->isa('$_') or do { package $_ };" }
+		@{$self->{fake}};
+
+	return <<"END_MODULES";
+use strict;${load}
+use Class::Adapter ();
+
+BEGIN {
+	\@${pkg}::ISA = 'Class::Adapter';${namespaces}
+}
+END_MODULES
 }
-
-
 
 sub _make_new { <<"END_NEW" }
 sub new {
@@ -338,16 +379,18 @@ END_METHOD
 
 sub _make_OBJECT { <<"END_OBJECT" }
 sub isa {
-	shift->_OBJECT_->isa(\@_);
+	ref(\$_[0])
+	? shift->_OBJECT_->isa(\@_)
+	: shift->isa(\@_);
 }
 
 sub can {
-	shift->_OBJECT_->can(\@_);
+	ref(\$_[0])
+	? shift->_OBJECT_->can(\@_)
+	: shift->can(\@_);
 }
 END_OBJECT
 
-
-
 sub _make_ISA {
 	my $self = shift;
 	my @lines = (
@@ -357,6 +400,11 @@ sub _make_ISA {
 		"}\n",
 		"\n",
 		"sub can {\n",
+		# If we are pretending to be a fake ISA, and we get a can call,
+		# we should try to require the module (even if it doesn't exist)
+		# so that we can provide an accurate answer in the case where
+		# we are faking a module that exists.
+		( map { "\trequire $_ unless $_->isa('UNIVERSAL');\n" } @{$self->{fake}} ),
 		"\treturn 1 if \$_[0]->SUPER::can(\$_[1]);\n",
 		( map { "\treturn 1 if $_->can(\$_[1]);\n" } @_ ),
 		"\treturn undef;\n",
@@ -370,10 +418,10 @@ sub _make_ISA {
 sub _make_AUTOLOAD { my $pub = $_[2] ? 'and substr($method, 0, 1) ne "_"' : ''; return <<"END_AUTOLOAD" }
 sub AUTOLOAD {
 	my \$self     = shift;
-	my (\$method) = \$$_[1]::AUTOLOAD =~ m/^.*::(.*)\$/s;
+	my (\$method) = \$$_[1]::AUTOLOAD =~ m/^.*::(.*)\\z/s;
 	unless ( ref(\$self) $pub) {
 		Carp::croak(
-			qq{Can't locate object method "\$method" via package "\$self" }
+			  qq{Can't locate object method "\$method" via package "\$self" }
 			. qq{(perhaps you forgot to load "\$self")}
 		);
 	}
@@ -381,14 +429,12 @@ sub AUTOLOAD {
 }
 
 sub DESTROY {
-	my \$object = \$_[0]->_OBJECT_;
-	\$object->DESTROY if \$object->can('DESTROY');
+	if ( defined \$_[0]->{OBJECT} and \$_[0]->{OBJECT}->can('DESTROY') ) {
+		undef \$_[0]->{OBJECT};
+	}
 }
 END_AUTOLOAD
 
-
-
-
 1;
 
 =pod
@@ -403,7 +449,7 @@ For other issues, contact the author.
 
 =head1 AUTHOR
 
-Adam Kennedy E<lt>adamk@cpan.orgE<gt>, L<http://ali.as/>
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
@@ -411,7 +457,8 @@ L<Class::Adapter>, L<Class::Adapter::Clear>
 
 =head1 COPYRIGHT
 
-Copyright 2005 Adam Kennedy.
+Copyright 2005 - 2009 Adam Kennedy.
+
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
